#include "ELF64_Section.h"

ELF64_Section::ELF64_Section()
	: hashId(0), programIndex(EHS_UINT_16_MAX + 1), nameOffset(0), type(0), flags(0), vAddr(0), segmentOffset(0), associatedIndex(0),
	info(0), align(0), entrySize(0), data(ehs::Endianness::LE)
{
}

ELF64_Section::ELF64_Section(ehs::Str_8 id, ehs::UInt_32 type, ehs::UInt_64 flags, ehs::UInt_64 align)
	: hashId(id.Hash_64()), id((ehs::Str_8&&)id), programIndex(EHS_UINT_16_MAX + 1), nameOffset(0), type(type), flags(flags), vAddr(0),
	segmentOffset(0), associatedIndex(0), info(0), align(align), entrySize(0), data(ehs::Endianness::LE)
{

}

ELF64_Section::ELF64_Section(ELF64_Section&& sect) noexcept
	: hashId(sect.hashId), id((ehs::Str_8&&)sect.id), programIndex(sect.programIndex), nameOffset(sect.nameOffset),
	type(sect.type), flags(sect.flags), vAddr(sect.vAddr), segmentOffset(sect.segmentOffset),
	associatedIndex(sect.associatedIndex), info(sect.info), align(sect.align), entrySize(sect.entrySize),
	data((ehs::Serializer<ehs::UInt_64>&&)sect.data)
{
	sect.hashId = 0;
	sect.programIndex = EHS_UINT_16_MAX + 1;
	sect.nameOffset = 0;
	sect.type = 0;
	sect.flags = 0;
	sect.vAddr = 0;
	sect.segmentOffset = 0;
	sect.associatedIndex = 0;
	sect.info = 0;
	sect.align = 0;
	sect.entrySize = 0;
}

ELF64_Section::ELF64_Section(const ELF64_Section& sect)
	: hashId(sect.hashId), id(sect.id), programIndex(EHS_UINT_16_MAX + 1), nameOffset(0), type(sect.type),
	flags(sect.flags), vAddr(sect.vAddr), segmentOffset(sect.segmentOffset), associatedIndex(sect.associatedIndex),
	info(sect.info), align(sect.align), entrySize(sect.entrySize), data(sect.data)
{
}

ELF64_Section& ELF64_Section::operator=(ELF64_Section&& sect) noexcept
{
	if (this == &sect)
		return *this;

	hashId = sect.hashId;
	id = (ehs::Str_8&&)sect.id;
	programIndex = sect.programIndex;
	nameOffset = sect.nameOffset;
	type = sect.type;
	flags = sect.flags;
	vAddr = sect.vAddr;
	segmentOffset = sect.segmentOffset;
	associatedIndex = sect.associatedIndex;
	info = sect.info;
	align = sect.align;
	entrySize = sect.entrySize;
	data = (ehs::Serializer<ehs::UInt_64>&&)sect.data;

	sect.hashId = 0;
	sect.programIndex = EHS_UINT_16_MAX + 1;
	sect.nameOffset = 0;
	sect.type = 0;
	sect.flags = 0;
	sect.vAddr = 0;
	sect.segmentOffset = 0;
	sect.associatedIndex = 0;
	sect.info = 0;
	sect.align = 0;
	sect.entrySize = 0;
	sect.data = {ehs::Endianness::LE};
	
	return *this;
}

ELF64_Section& ELF64_Section::operator=(const ELF64_Section& sect)
{
	if (this == &sect)
		return *this;

	hashId = sect.hashId;
	id = sect.id;
	programIndex = EHS_UINT_16_MAX + 1;
	nameOffset = 0;
	type = sect.type;
	flags = sect.flags;
	vAddr = sect.vAddr;
	segmentOffset = sect.segmentOffset;
	associatedIndex = sect.associatedIndex;
	info = sect.info;
	align = sect.align;
	entrySize = sect.entrySize;
	data = sect.data;

	return *this;
}

ehs::UInt_64 ELF64_Section::GetHashId() const
{
	return hashId;
}

ehs::Str_8 ELF64_Section::GetId() const
{
	return id;
}

ehs::UInt_32 ELF64_Section::GetType() const
{
	return type;
}

ehs::UInt_64 ELF64_Section::GetFlags() const
{
	return flags;
}

ehs::UInt_64 ELF64_Section::GetVirtualAddress() const
{
	return vAddr;
}

ehs::UInt_64 ELF64_Section::GetAlignment() const
{
	return align;
}

void ELF64_Section::SetData(ehs::Serializer<ehs::UInt_64> newData)
{
	data = (ehs::Serializer<ehs::UInt_64>&&)newData;
}

ehs::Serializer<ehs::UInt_64>& ELF64_Section::GetData()
{
	return data;
}

void ELF64_Section::Serialize(ehs::Serializer<ehs::UInt_64>& inData) const
{
	inData.Write(nameOffset);
	inData.Write(type);
	inData.Write(flags);
	inData.Write(vAddr);
	inData.Write(segmentOffset);
	inData.Write(data.Size());
	inData.Write(associatedIndex);
	inData.Write(info);
	inData.Write(align);
	inData.Write(entrySize);
}
